from tkinter import *
from data_bevaring import *
from data_oprindelig import oprindelig_materialeliste
from klasser import *
from dialog_bokse import *


# Materialelisten, der i allerførste omgang udfyldes på baggrund af
# oprindelig_materialeliste i filen data_oprindelig.py.
materialeliste = []


class Application(Frame):
    """
    Dette program er et mini-administrationssystem til et folkebibliotek.
    Omdrejningspunktet er en materialeliste, der kan vises i sin fuldstændige
    form samt søges i via både tekststrenge og ID-numre.
    Materialerne består i bøger, film, lp'er og spil, som hver især er objekter.
    Alle materialerne kan afleveres, lånes, reserveres og slettes, og nye
    materialer kan oprettes.
    Derudover kan materialelisten som samlet størrelse gemmes fra gang til gang,
    og den oprindelige materialeliste kan til enhver tid gendannes.
    """

    """
    SØGNING
    """
    # Når der trykkes på knappen med teksten "Søg" kaldes denne funktion.
    # Af hensyn til ønsket om genvisning af sidste visning i opdateret form,
    # sættes seneste_vis_idnr til "None". Af samme hensyn sættes "seneste_soege
    # _tekst" til det, der lige nu står i søge-entry-feltet. (Der er en
    # længere forklaring heraf nederst i nærværende fil, hvor omtalte variabler
    # deklareres og initieres.)
    # En besked om søgningen sendes til konsollen, mens søge-teksten sendes
    # videre til "soeg_i_listen" lige nedenunder.
    def soeg_i_listen_valgt(self):
        self.seneste_vis_idnr = None
        soege_tekst = self.soeg_entry.get()
        self.seneste_soege_tekst = soege_tekst
        print("Søgning foretaget på: '" + soege_tekst + "'.")
        self.soeg_i_listen(soege_tekst)

    # Denne funktion kaldes bl.a. af ovenstående funktion, dvs. når der trykkes
    # på søge-knappen. Den kaldes dog også andre steder fra, hvor ønsket er at
    # genvise den sidste visning i tekstfeltet. Fra de steder er det dog den
    # seneste_soege_tekst, der sendes med. Dén variabel sættes i fx
    # "vis_hele_listen" til den tomme streng, "", hvilket, som vi kan se her,
    # netop får hele listen til at blive skrevet ud i tekstfeltet.
    def soeg_i_listen(self, soege_tekst):
        if soege_tekst == "":
            self.vis_hele_listen()
        else:
            # Er der derimod en søgetekst, så slettes indholdet i tekstfeltets
            # liste, hvorefter materialelisten søges igennem via de forskellige
            # objekttypers match_noget()-funktioner.
            self.listGui.delete('1.0', END)
            materiale_fundet = False
            for materiale in materialeliste:
                if materiale.match_noget(soege_tekst):
                    # Betingelsen opfyldes kun ved første match, hvormed teksten
                    # indsættes øverst i tekstfeltet, når altså listGui bliver
                    # til et tekstfelt.
                    if not materiale_fundet:
                        self.listGui.insert(INSERT, "\n  Søgeresultater på '"
                                            + soege_tekst + "':\n\n")
                    self.listGui.insert(INSERT, materiale.til_streng() + "\n")
                    # Her sørges for, at "Søgeresultater på..." kun skrives én
                    # gang.
                    materiale_fundet = True
            # Hvis intet blev fundet, er "not False" => True, hvormed denne
            # betingelse altså opfyldes, og beskeden skrives i tekstfeltet.
            if not materiale_fundet:
                self.listGui.insert(INSERT, "\n  Ingen søgeresultater på '"
                                    + soege_tekst + "'.")

    """
    VISNING AF LISTEN I OPDATERET OG KOMPLET FORM
    """
    def vis_hele_listen_valgt(self):
        print("Hele listen vises.")
        self.vis_hele_listen()

    # Bruges kun ifm. tryk på "Vis hele listen".
    def vis_hele_listen(self):
        self.seneste_vis_idnr = None
        self.seneste_soege_tekst = ""
        self.listGui.delete('1.0', END)
        for materiale in materialeliste:
            self.listGui.insert(INSERT, materiale.til_streng() + "\n")

    """
    VISNING UD FRA ID
    """
    def vis_id_valgt(self):
        idnr = self.id_entry.get()
        self.seneste_vis_idnr = idnr
        print("Materialet med ID der skal vises: " + idnr + ".")
        self.vis_id()

    def vis_id(self):
        idnr = self.seneste_vis_idnr
        materialet_fundet = False
        for materiale in materialeliste:
            # idnr er en integer i hvert objekt, hvorfor det konverteres til en
            # streng, så det kan sammenlignes med den streng, der ligger i
            # variablen self.seneste_vis_idnr
            if idnr == str(materiale.idnr):
                materialet_fundet = True
                self.listGui.delete('1.0', END)
                self.listGui.insert(INSERT, "\n  Materialet med ID "
                                    + idnr + " vises:\n\n")
                self.listGui.insert(INSERT, materiale.til_streng() + "\n")
                # Der skulle ikke gerne kunne dukke flere op, så vi breaker her
                # af effektivitetshensyn.
                break
        if not materialet_fundet:
            dialog_id_findes_ikke()

    """
    AFLEVERING
    """
    def aflever_valgt(self):
        idnr = self.id_entry.get()
        print("ID der skal afleveres: " + idnr + ".")
        materialet_fundet = False

        for materiale in materialeliste:
            if idnr == str(materiale.idnr):
                materialet_fundet = True
                if materiale.aflever_muligt():
                    materiale.aflever()
                    print("Materialet med ID " + idnr
                          + " er blevet afleveret.")
                    # Afleveringen formindsker antal udlån med 1,
                    # hvorfor visningen af listen opdateres. Da ID for hvert
                    # materiale er en unik størrelse, breakes der derefter.
                    if self.seneste_vis_idnr is None:
                        self.soeg_i_listen(self.seneste_soege_tekst)
                    else:
                        self.vis_id()
                    break
                else:
                    messagebox.showwarning("Servicemeddelelse",
                                           "Kan ikke afleveres, da ingen "
                                           "materialer med ID " + idnr
                                           + " er udlånt.")
        if not materialet_fundet:
            dialog_id_findes_ikke()

    """
    ALMINDELIG UDLÅN (DVS. IKKE PÅ BAGGRUND AF RESERVATION)
    """
    def udlaan_valgt(self):
        idnr = self.id_entry.get()
        print("ID der skal lånes: " + idnr + ".")
        materialet_fundet = False

        for materiale in materialeliste:
            if idnr == str(materiale.idnr):
                materialet_fundet = True
                if materiale.udlaan_muligt():
                    materiale.udlaan()
                    print("Materialet med ID " + idnr
                          + " er blevet udlånt.")
                    # Udlåningen forøger antal udlån med 1,
                    # hvorfor visningen af listen opdateres. Da ID for hvert
                    # materiale er en unik størrelse, breakes der derefter.
                    if self.seneste_vis_idnr is None:
                        self.soeg_i_listen(self.seneste_soege_tekst)
                    else:
                        self.vis_id()
                    break
                else:
                    if messagebox.askyesno("Servicemeddelelse",
                                           "Kan ikke udlånes, da alle "
                                           "materialer med ID " + idnr
                                           + " er udlånt eller reserverede.\n"
                                             "\nØnsker du at reservere "
                                             "materialet?"):
                        print("ID der skal reserveres: " + idnr + ".")
                        print("Materialet med ID " + idnr + " er blevet "
                                                            "reserveret.")
                        materiale.reserver()
                        # Reservationen forøger antal reservationer med 1,
                        # hvorfor visningen af listen opdateres.
                        if self.seneste_vis_idnr is None:
                            self.soeg_i_listen(self.seneste_soege_tekst)
                        else:
                            self.vis_id()
        if not materialet_fundet:
            dialog_id_findes_ikke()

    """
    RESERVATION OG LÅN PÅ BAGGRUND AF RESERVATION
    """
    def reservation_valgt(self):
        idnr = self.id_entry.get()
        print("ID der skal reserveres: " + idnr + ".")
        materialet_fundet = False

        for materiale in materialeliste:
            if idnr == str(materiale.idnr):
                materialet_fundet = True
                materiale.reserver()
                print("Materialet med ID " + idnr + " er blevet reserveret.")
                # Reserveringen forøger antal reservationer med 1,
                # hvorfor visningen af listen opdateres. Da ID for hvert
                # materiale er en unik størrelse, breakes der derefter.
                # Af hensyn til genvisningsønsket kommer denne if-else-kæde, der
                # gentages et par gange i de følgende funktioner. Hvis seneste_
                # vis_idnr er "None", så er der ikke blevet vist via ID som det
                # seneste. Derfor vises i det tilfælde vha. soeg_i_listen med
                # den seneste søgetekst (som evt. er en tom streng, hvormed
                # hele listen i stedet vil blive vist). Er seneste_vis_idnr ikke
                # None, så er det visningen på baggrund af ID, der skal vises.
                if self.seneste_vis_idnr is None:
                    self.soeg_i_listen(self.seneste_soege_tekst)
                else:
                    self.vis_id()
                if materiale.for_lille_udbud():
                    messagebox.showinfo("Stor efterspørgsel på ID " + idnr,
                                        "Overvej at indkøbe flere eksemplarer"
                                        " af materialet med ID "
                                        + idnr + ".")
                break
        if not materialet_fundet:
            dialog_id_findes_ikke()

    def udlaanreservation_valgt(self):
        idnr = self.id_entry.get()
        print("ID der skal lånes på baggrund af reservation: " + idnr + ".")
        materialet_fundet = False

        for materiale in materialeliste:
            if idnr == str(materiale.idnr):
                materialet_fundet = True
                if materiale.laan_reservation_muligt():
                    materiale.fjern_reservation()
                    materiale.udlaan()
                    print("Materialet med ID " + idnr
                          + " er blevet udlånt på baggrund af reservation.")
                    # Udlåningen på baggrund af reservation forøger antal udlån
                    # med 1 og formindsker antal reservationer med 1,
                    # hvorfor visningen af listen opdateres. Da ID for hvert
                    # materiale er en unik størrelse, breakes der derefter.
                    if self.seneste_vis_idnr is None:
                        self.soeg_i_listen(self.seneste_soege_tekst)
                    else:
                        self.vis_id()
                    break
                elif materiale.reservationer == 0:
                    messagebox.showinfo("Servicemeddelelse",
                                        "Materialet med ID " + idnr
                                        + " kan ikke udlånes på baggrund af en"
                                          " reservation, da det ikke er "
                                          "reserveret.")
                    break
                else:
                    messagebox.showinfo("Servicemeddelelse",
                                        "Materialet med ID " + idnr
                                        + " kan ikke udlånes, da ingen"
                                          " eksemplarer haves hjemme.")
                    break
        if not materialet_fundet:
            dialog_id_findes_ikke()

    """
    SLETNING
    """
    def slet_valgt(self):
        idnr = self.id_entry.get()
        print("ID der skal slettes: " + idnr + ".")
        materialet_fundet = False
        # Her benyttes en indekstæller, så sletningen kan foretages direkte via
        # "del" på baggrund af det allerede én gang fundne materiale.
        # (Brug af ".remove()" ville gennemsøge listen endnu engang.)
        indeks = 0

        for materiale in materialeliste:
            if idnr == str(materiale.idnr):
                materialet_fundet = True
                if messagebox.askyesno("Advarsel!",
                                       "Slet materialet med ID " +
                                       idnr + " ?\n\nSletningen kan "
                                              "ikke gøres om."):
                    del materialeliste[indeks]
                    # Et objekt er blevet slettet, hvorfor visningen af listen
                    # opdateres. Dernæst informeres brugeren om hændelsen, og
                    # der breakes, da ID-numrene er unikke.
                    if self.seneste_vis_idnr is None:
                        self.soeg_i_listen(self.seneste_soege_tekst)
                    # Her er kommet en ekstra stump kode ind i forhold til de
                    # lignende dele andre steder. Hele listen skal vises, hvis
                    # senest viste ting var det element, der netop er blevet
                    # slettet, og hvis det blev vist via "Vis"-knappen, for
                    # ellers vil der komme en irrelevant (men sand)
                    # fejlmeddelelse frem om, at det søgte ID ikke findes.
                    elif self.seneste_vis_idnr != idnr:
                        self.vis_id()
                    else:
                        self.vis_hele_listen()
                    messagebox.showinfo("Servicemeddelelse",
                                        "Materialet med ID "
                                        + idnr + " er blevet slettet.")
                    break
                else:
                    messagebox.showinfo("Servicemeddelelse",
                                        "Materialet med ID "
                                        + idnr + " er IKKE blevet slettet.")
                    break
            indeks += 1
        if not materialet_fundet:
            dialog_id_findes_ikke()

    """
    OPRETTELSE AF NYT MATERIALE
    """
    def opret_nyt_materiale_valgt(self):
        # Et Toplevel-objekt oprettes.
        self.window = Toplevel()
        self.window.title("Oprettelse af nyt materiale")

        # Måden jeg opretter mit nye vindue på gør, at åbning af flere af dem på
        # én gang og først efterfølgende forsøg på oprettelser af nye materialer
        # ved hjælp af dem giver fejlmeddelelser. Derfor tager jeg fat i vinduet
        # og bringer det i forgrunden med funktionen herunder, så der simpelthen
        # ikke kan trykkes på "opret nyt materiale"-knappen, førend det lille
        # vindue atter er blevet lukket (via oprettelse eller fortryd.)
        self.window.grab_set()

        # Hvert andet af disse bruges som mellemrum, så vinduet bliver pænt og
        # overskueligt.
        # I hvert frame bruges dernæst grids, så elementerne i hvert grid kan
        # komme til at stå pænt under hinanden.
        self.frame1 = Frame(self.window)
        self.frame2 = Frame(self.window)
        self.frame3 = Frame(self.window)
        self.frame4 = Frame(self.window)
        self.frame5 = Frame(self.window)
        self.frame6 = Frame(self.window)
        self.frame7 = Frame(self.window)

        # Grafisk mellemrum.
        Label(self.frame1).grid(row=0, column=0)
        self.frame1.grid(row=0, column=0)

        # Radioknap, hvor materialetypen skal vælges.
        # Den sættes i første omgang til "Bog".
        # Hver gang der trykkes på en af knapperne, kaldes funktionen længere
        # nede med navnet "midlertidigt_materialevalg". Dén funktion ændrer
        # labelteksterne på de to nederste labels, da disse skal være
        # individuelle fra materialetype til materialetype.
        # (Fx Forfatter/Instruktør/Kunstner/Udgiver.)
        self.radioknap = IntVar()
        self.radioknap.set(1)
        Label(self.frame2, text="Vælg materialetype:", width=20).grid(row=0,
                                                                 column=0)
        Radiobutton(self.frame2, text="Bog", variable=self.radioknap, value=1,
                    command=self.midlertidigt_materialevalg)\
            .grid(row=0, column=1)
        Radiobutton(self.frame2, text="Film", variable=self.radioknap, value=2,
                    command=self.midlertidigt_materialevalg)\
            .grid(row=0, column=2)
        Radiobutton(self.frame2, text="Lp", variable=self.radioknap, value=3,
                    command=self.midlertidigt_materialevalg)\
            .grid(row=0, column=3)
        Radiobutton(self.frame2, text="Spil", variable=self.radioknap, value=4,
                    command=self.midlertidigt_materialevalg, anchor=W, width=7)\
            .grid(row=0, column=4)
        self.frame2.grid(row=1, column=0)

        # Grafisk mellemrum.
        Label(self.frame3).grid(row=0, column=0)
        self.frame3.grid(row=2, column=0)

        # Her har vi 5 labels (og tilhørende entry-felter), hvor radioknappen
        # ovenover via funktionen "midlertidigt_materialevalg" kan ændre på
        # label4 og label5's tekst, da de skal være materialetype/objekttype-
        # afhængige.
        Label(self.frame4, text="Titel:", width=10).grid(row=0, column=0)
        self.entry1 = Entry(self.frame4, bd=5, width=35)
        self.entry1.grid(row=0, column=1)
        Label(self.frame4, text="Antal:", width=10).grid(row=1, column=0)
        self.entry2 = Entry(self.frame4, bd=5, width=35)
        self.entry2.grid(row=1, column=1)
        Label(self.frame4, text="Årstal:", width=10).grid(row=2, column=0)
        self.entry3 = Entry(self.frame4, bd=5, width=35)
        self.entry3.grid(row=2, column=1)
        label4 = Label(self.frame4, text="Forfatter:", width=10)
        label4.grid(row=3, column=0)
        self.entry4 = Entry(self.frame4, bd=5, width=35)
        self.entry4.grid(row=3, column=1)
        label5 = Label(self.frame4, text="Sider:", width=10)
        label5.grid(row=4, column=0)
        self.entry5 = Entry(self.frame4, bd=5, width=35)
        self.entry5.grid(row=4, column=1)
        self.frame4.grid(row=3, column=0)

        # Grafisk mellemrum.
        Label(self.frame5).grid(row=0, column=0)
        self.frame5.grid(row=4, column=0)

        # 2 knapper, der hhv. forsøger at oprette det nye materiale eller
        # fortryder, hvorved vinduet lukkes.
        Button(self.frame6, text="Opret materiale", command=self.tjek_vaerdier)\
            .grid(row=0, column=0)
        Button(self.frame6, text="Fortryd",
               command=self.window.destroy).grid(row=0, column=1)
        self.frame6.grid(row=5, column=0)

        # Grafisk mellemrum.
        Label(self.frame7).grid(row=0, column=0)
        self.frame7.grid(row=6, column=0)

    def midlertidigt_materialevalg(self):
        # Radioknappens værdi hentes (1/2/3/4),
        # og self.valg-variablen bærer den.
        self.valg = self.radioknap.get()
        # Jeg får fat i de 2 labels ved hjælp af global-deklareringen.
        global label4
        global label5

        # Radioknappens værdi afgør via denne forholdsvis manuelle tildeling,
        # hvad der skal stå på label4 og label5.
        # Den sidste "elif..." kunne sagens erstattes af "else:" og så ingen
        # betingelse efterfølgende. Der er dog netop ingen grund til en
        # catch-all fanger her, da værdien netop må være 4, hvis den ikke er 1,
        # 2 eller 3.
        if self.valg == 1:
            tekst4 = "Forfatter"
            tekst5 = "Sider"
        elif self.valg == 2:
            tekst4 = "Instruktør"
            tekst5 = "Varighed"
        elif self.valg == 3:
            tekst4 = "Kunstner"
            tekst5 = "Varighed"
        elif self.valg == 4:
            tekst4 = "Udgiver"
            tekst5 = "Konsol"

        # tekst4 og tekst5 har fået nyt indhold (der dog muligvis er det samme
        # som tidligere), hvorfor de 2 labels nu geninitieres og -placeres.
        label4 = Label(self.frame4, text=tekst4 + ":", width=10).grid(row=3,
                                                                      column=0)
        label5 = Label(self.frame4, text=tekst5 + ":", width=10).grid(row=4,
                                                                      column=0)

    # Denne funktion kaldes, når der trykkes på "opret nyt materiale-knappen".
    # Den tjekker værdiernes korrekthed i følgende rækkefølge og fortsætter hver
    # gang først, når hvert punkt er opfyldt:
    # 1. Der skal stå noget i hvert felt.
    # 2. Værdierne i felt 2 og 3 (antal og årstal) skal være heltal.
    # 3. De hele tal i felt 2 og 3 skal være positive.
    # Hvis der er tale om et et spil, godkendes værdierne.
    # 4. Hvis der ikke er tale om et spil: Der skal være et helt tal i det
    #    nederste felt (sider/varighed)
    # Værdierne godkendes.
    # Når værdierne er blevet godkendt, sendes de via en liste videre til
    # funktionen "opret_nyt_materiale".
    def tjek_vaerdier(self):
        self.valg = self.radioknap.get()
        e1 = self.entry1.get()
        e2 = self.entry2.get()
        e3 = self.entry3.get()
        e4 = self.entry4.get()
        e5 = self.entry5.get()

        if e1 == "" or e2 == "" or e3 == "" or e4 == "" or e5 == "":
            messagebox.showwarning("Servicemeddelse", "Mindst et af felterne er"
                                                      " tomt.\n\nPrøv igen.")
        else:
            try:
                e2 = int(e2)
                e3 = int(e3)
            except ValueError:
                messagebox.showwarning("Servicemeddelse",
                                       'Du har ikke skrevet hele tal ud for "'
                                       'antal" og/eller "årstal".'
                                       '\n\nPrøv igen.')
            else:
                if e2 > 0 and e3 > 0:
                    if self.valg == 1 or self.valg == 2 or self.valg == 3:
                        try:
                            e5 = int(e5)
                        except ValueError:
                            messagebox.showwarning("Servicemeddelse",
                                                   "Angivelsen i det nederste "
                                                   "felt skal være et helt "
                                                   "tal.\n\nPrøv igen.")
                        else:
                            print("Funktion kaldes, som opretter materialet.")
                            self.opret_nyt_materiale([self.valg, e1, e2, e3, e4,
                                                      e5])
                            self.window.destroy()
                            messagebox.showinfo("Servicemeddelelse",
                                                "Materialet er "
                                                "blevet oprettet.")
                    elif self.valg == 4:
                        print("Funktion kaldes, som opretter materialet.")
                        self.opret_nyt_materiale([self.valg, e1, e2, e3, e4,
                                                  e5])
                        self.window.destroy()
                        messagebox.showinfo("Servicemeddelelse",
                                            "Materialet er "
                                            "blevet oprettet.")
                else:
                    messagebox.showwarning("Servicemeddelse",
                                           'Du har ikke skrevet positive tal '
                                           'ud for "antal" og/eller "årstal".'
                                           '\n\nPrøv igen.')

    # Her oprettes det nye materiale ud fra
    # en liste af allerede godkendte værdier.
    def opret_nyt_materiale(self, nyt_objekts_vaerdier):
        liste = nyt_objekts_vaerdier

        # Materialelistens højeste ID-nummer findes
        max_id = 0
        for materiale in materialeliste:
            if materiale.idnr > max_id:
                max_id = materiale.idnr

        # Den medsendte listes første værdi består i radioknappens værdi.
        # 1 betyder Bog, 2 betyder Film osv. Derfor dannes objektet med
        # udgangspunkt i denne betingelse.
        # Objektet tildeles et ID-nummer, der er 1 højere end materialelistens
        # højeste ID-nummer. Derudover lægges værdierne i den medsendte liste
        # ind på de rigtige steder i de forskellige objektbyggere, da de
        # nødvendige konverteringer allerede er foretaget i funktionen
        # "tjek_værdier". De 2 rene nuller er antal udlån og reservationer.
        if liste[0] == 1:
            nyt_objekt = Bog(max_id + 1, liste[1], liste[2], 0, 0,
                             liste[3], liste[4], liste[5])
        elif liste[0] == 2:
            nyt_objekt = Film(max_id + 1, liste[1], liste[2], 0, 0,
                              liste[3], liste[4], liste[5])
        elif liste[0] == 3:
            nyt_objekt = Lp(max_id + 1, liste[1], liste[2], 0, 0,
                            liste[3], liste[4], liste[5])
        elif liste[0] == 4:
            nyt_objekt = Spil(max_id + 1, liste[1], liste[2], 0, 0,
                              liste[3], liste[4], liste[5])

        # Det nye objekt lægges ind til sidst i materialelisten, hvorefter
        # hele listen vises.
        materialeliste.append(nyt_objekt)
        self.vis_hele_listen()
        print("Nyt materiale er blevet oprettet.")

    """
    FØRSTE OPRETTELSE AF DATA, NULSTILLING AF DATA OG BEVARELSE AF DATA
    """
    def udfyld_materialeliste(self):
        if findes_materialeliste():
            gemt_materialeliste = hent_gemt_materialeliste()
            # Element for element flyttes fra den arkiverede liste over i den
            # tomme materialeliste.
            for element in gemt_materialeliste:
                materialeliste.append(element)
        else:
            self.nulstil_materialeliste()

    def gem(self):
        gem_materialeliste(materialeliste)
        messagebox.showinfo("Servicemeddelelse",
                            "Alle ændringer er blevet gemt.")

    def gem_og_luk_program(self):
        gem_materialeliste(materialeliste)
        self.quit()

    def nulstil_materialeliste_valgt(self):
        if messagebox.askyesno("Advarsel!",
                               "Vil du sætte materialelistens indhold tilbage"
                               " til udgangspunktet?\n\nNulstillingen kan "
                                      "ikke gøres om."):
            self.nulstil_materialeliste()
            self.vis_hele_listen()
            messagebox.showinfo("Servicemeddelelse",
                                "Materialelisten er blevet nulstillet.")

    def nulstil_materialeliste(self):
        materialeliste.clear()
        for element in oprindelig_materialeliste:
            materialeliste.append(element)

    """
    HOVEDVINDUETS FRAME, LABELS, ENTRY-FELTER, KNAPPER OG TEKSTFELT (HVORI
    LISTEN / FORSKELLIGE DELE AF LISTEN VISES)
    """
    def create_widgets(self):
        frame = Frame(self)
        self.winfo_toplevel().title("Biblioteksdatabasen")

        # Definition af søgeinput-feltet.
        self.L1 = Label(frame, text="Søgning: ", height=2)
        self.L1.pack(side=LEFT)
        self.soeg_entry = Entry(frame, bd=5, width=24)
        self.soeg_entry.pack(side=LEFT)

        # Definition og mapping af søg-knappen.
        self.sogKnap = Button(frame, text="Søg")
        self.sogKnap["command"] = self.soeg_i_listen_valgt
        self.sogKnap.pack({"side": "left"})

        # Definition og mapping af vis hele listen-knappen.
        self.visListe = Button(frame, text="Vis hele listen")
        self.visListe["command"] = self.vis_hele_listen_valgt
        self.visListe.pack({"side": "left"})

        # Grafisk mellemrum.
        self.L1 = Label(frame, width=4)
        self.L1.pack(side=LEFT)

        # Definition af ID-inputfeltet (til vis, aflevering, udlån, reservation,
        # udlån på baggrund af reservation og sletning).
        self.L1 = Label(frame, text="Indtast ID: ")
        self.L1.pack(side=LEFT)
        self.id_entry = Entry(frame, bd=5, width=8)
        self.id_entry.pack(side=LEFT)

        # Definition og mapping af vis-knap.
        self.visKnap = Button(frame, text="Vis")
        self.visKnap["command"] = self.vis_id_valgt
        self.visKnap.pack({"side": "left"})

        # Definition og mapping af aflevering-knap.
        self.afleverKnap = Button(frame, text="Aflever")
        self.afleverKnap["command"] = self.aflever_valgt
        self.afleverKnap.pack({"side": "left"})

        # Definition og mapping af udlån-knap.
        self.udlaanKnap = Button(frame, text="Udlån")
        self.udlaanKnap["command"] = self.udlaan_valgt
        self.udlaanKnap.pack({"side": "left"})

        # Definition og mapping af reservation-knap.
        self.reservationKnap = Button(frame, text="Reservér")
        self.reservationKnap["command"] = self.reservation_valgt
        self.reservationKnap.pack({"side": "left"})

        # Definition og mapping af udlån reservation-knap.
        self.udlaanreservationKnap = Button(frame, text="Udlån reserveret "
                                                      "materiale")
        self.udlaanreservationKnap["command"] = self.udlaanreservation_valgt
        self.udlaanreservationKnap.pack({"side": "left"})

        # Definition og mapping af slet-knap.
        self.sletKnap = Button(frame, text="Slet")
        self.sletKnap["fg"] = "red"
        self.sletKnap["command"] = self.slet_valgt
        self.sletKnap.pack({"side": "left"})

        # Grafisk mellemrum.
        self.L1 = Label(frame, width=4)
        self.L1.pack(side=LEFT)

        # Definition og mapping af opret nyt materiale-knap.
        self.opretKnap = Button(frame, text="Opret nyt materiale")
        self.opretKnap["command"] = self.opret_nyt_materiale_valgt
        self.opretKnap.pack({"side": "left"})

        # Definition og mapping af nulstillingsknap.
        self.nulstillingKnap = Button(frame, text="Gendan oprindelig liste")
        self.nulstillingKnap["fg"] = "red"
        self.nulstillingKnap["command"] = self.nulstil_materialeliste_valgt
        self.nulstillingKnap.pack({"side": "left"})

        # Grafisk mellemrum.
        self.L1 = Label(frame, width=4)
        self.L1.pack(side=LEFT)

        # Definition af gem-knap.
        self.gemKnap = Button(frame, text="Gem", width=5)
        self.gemKnap["fg"] = "red"
        self.gemKnap["command"] = self.gem
        self.gemKnap.pack({"side": "left"})

        # Definition af gem og luk-knap.
        self.gemOgLukKnap = Button(frame, text="Gem og luk", width=9)
        self.gemOgLukKnap["fg"] = "red"
        self.gemOgLukKnap["command"] = self.gem_og_luk_program
        self.gemOgLukKnap.pack({"side": "left"})

        # Definition af luk-knap.
        self.lukKnap = Button(frame, text="Luk", width=9)
        self.lukKnap["fg"] = "red"
        self.lukKnap["command"] = self.quit
        self.lukKnap.pack({"side": "left"})

        # Tekstfeltet herunder forsynes med en scrollbar og placeres under de
        # øvrige elementer. På tekstfeltets linjer udskrives
        # materialelistens elementer vha. en for-løkke et for et via
        # til_streng-funktionerne i de forskellige klasser.
        self.S = Scrollbar(self)
        self.listGui = Text(self, height=30, width=168)
        self.S.pack(side=RIGHT, fill=Y)
        self.listGui.pack(side=LEFT, fill=Y)
        self.S.config(command=self.listGui.yview)
        self.listGui.config(yscrollcommand=self.S.set)
        for materiale in materialeliste:
            self.listGui.insert(INSERT, materiale.til_streng()+"\n")
        frame.pack()
        self.listGui.pack({"side": "bottom"})

    # Application(master=root) i bunden af denne fil laver et objekt ud fra
    # konstruktøren herunder, hvilket således sker kort efter, at programmet
    # sættes i gang. Konstruktøren sørger med "self.udfyld_materialelisten" (og
    # en hel del underfunktioner) for, at materialelisten har et indhold: Ved
    # allerførste start udfyldes materialelisten med de "oprindelige data" fra
    # filen data_oprindelig.py, ellers hentes dataene, som de var ved sidste
    # kørsel af programmet.
    # self.create_widgets() laver dernæst de forskellige widgets i hovedvinduet.
    def __init__(self, master=None):
        Frame.__init__(self, master)
        self.pack()
        self.udfyld_materialeliste()
        self.create_widgets()
        # Variablerne herunder indføres, så den senest valgte visning (søgning /
        # vis ID / vis hele listen) genvises i opdateret format (uanset hvad der
        # efterfølgende er skrevet i søgefeltet eller ID-feltet).
        # Dermed bliver det nemmere for brugeren at følge med i, at det
        # efterspurgte faktisk sker ifm. aflevering, udlåning, reservering og
        # sletning, medmindre brugeren selvfølgelig har valgt ikke at se på
        # de relevante materialer under omtalte handlinger, men dette
        # vil jeg netop lade være op til brugeren, der således altid ser det,
        # han/hun beder om. (Se uddybning heraf i de to øverste funktioner i
        # nærværende fil).
        self.seneste_soege_tekst = ""
        self.seneste_vis_idnr = None


"""
PROGRAMMET STARTES
"""
root = Tk()
app = Application(master=root)
app.mainloop()
